
CREATE TYPE mpaa_rating AS ENUM (
    'G',
    'PG',
    'PG-13',
    'R',
    'NC-17'
);


CREATE DOMAIN year AS integer
	CONSTRAINT year_check CHECK (((VALUE >= 1901) AND (VALUE <= 2155)));

--SECUENCIAS
CREATE SEQUENCE actor_actor_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MAXVALUE
    NO MINVALUE
    CACHE 1;

CREATE SEQUENCE categoria_categoria_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MAXVALUE
    NO MINVALUE
    CACHE 1;

CREATE SEQUENCE ciudad_id_ciudad_seq
    START WITH 1
    INCREMENT BY 1
    NO MAXVALUE
    NO MINVALUE
    CACHE 1;
	
CREATE SEQUENCE cliente_cliente_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MAXVALUE
    NO MINVALUE
    CACHE 1;
	
CREATE SEQUENCE idioma_id_idioma_seq
    START WITH 1
    INCREMENT BY 1
    NO MAXVALUE
    NO MINVALUE
    CACHE 1;
	
CREATE SEQUENCE pais_id_pais_seq
    START WITH 1
    INCREMENT BY 1
    NO MAXVALUE
    NO MINVALUE
    CACHE 1;
	
CREATE SEQUENCE pelicula_pelicula_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MAXVALUE
    NO MINVALUE
    CACHE 1;

CREATE SEQUENCE personal_id_personal_seq
    START WITH 1
    INCREMENT BY 1
    NO MAXVALUE
    NO MINVALUE
    CACHE 1;
	
CREATE SEQUENCE sucursal_sucursal_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MAXVALUE
    NO MINVALUE
    CACHE 1;

	
--TABLAS	
CREATE TABLE actores (
    id_actor integer DEFAULT nextval('actor_actor_id_seq'::regclass) NOT NULL,
    nombre character varying(45) NOT NULL,
    apellido character varying(45) NOT NULL
);

CREATE TABLE actores_de_peliculas (
    id_actor smallint NOT NULL,
    id_pelicula smallint NOT NULL
);

CREATE TABLE alquileres (
    id_pelicula smallint NOT NULL,
    id_sucursal smallint NOT NULL,
    id_cliente smallint NOT NULL,
    fecha timestamp without time zone NOT NULL,
    id_personal smallint NOT NULL,
    fecha_devolucion timestamp without time zone
);

CREATE TABLE categorias (
    id_categoria integer DEFAULT nextval('categoria_categoria_id_seq'::regclass) NOT NULL,
    categoria character varying(25) NOT NULL
);

CREATE TABLE categorias_de_peliculas (
    id_pelicula smallint NOT NULL,
    id_categoria smallint NOT NULL
);

CREATE TABLE ciudades (
    id_ciudad integer DEFAULT nextval('ciudad_id_ciudad_seq'::regclass) NOT NULL,
    ciudad character varying(50) NOT NULL,
    id_pais smallint NOT NULL
);

CREATE TABLE clientes (
    id_cliente integer DEFAULT nextval('cliente_cliente_id_seq'::regclass) NOT NULL,
    id_sucursal smallint NOT NULL,
    nombre character varying(45) NOT NULL,
    apellido character varying(45) NOT NULL,
    email character varying(50),
    direccion character varying(50) NOT NULL,
    distrito character varying(20) NOT NULL,
    id_ciudad smallint NOT NULL,
    codigo_postal character varying(10),
    telefono character varying(20) NOT NULL,
    activo boolean DEFAULT true NOT NULL
);

CREATE TABLE idiomas (
    id_idioma integer DEFAULT nextval('idioma_id_idioma_seq'::regclass) NOT NULL,
    idioma character(20) NOT NULL
);

CREATE TABLE idiomas_de_peliculas (
    id_idioma smallint NOT NULL,
    id_pelicula smallint NOT NULL
);

CREATE TABLE inventario (
    id_pelicula smallint NOT NULL,
    id_sucursal smallint NOT NULL,
    cant_ejemplares smallint NOT NULL
);

CREATE TABLE pagos (
    id_pelicula_alquilo smallint NOT NULL,
    id_cliente_alquilo smallint NOT NULL,
    id_sucursal_alquilo smallint NOT NULL,
    id_personal_alquilo smallint NOT NULL,
    fecha_alquilo timestamp without time zone NOT NULL,
    id_personal_recibe_pago smallint NOT NULL,
    monto numeric(5,2) NOT NULL,
    fecha timestamp without time zone NOT NULL
);

CREATE TABLE paises (
    id_pais integer DEFAULT nextval('pais_id_pais_seq'::regclass) NOT NULL,
    pais character varying(50) NOT NULL
);

CREATE TABLE peliculas (
    id_pelicula integer DEFAULT nextval('pelicula_pelicula_id_seq'::regclass) NOT NULL,
    titulo character varying(255) NOT NULL,
    descripcion text,
    anio year,
    id_idioma_original smallint,
    duracion_alquiler smallint DEFAULT 3 NOT NULL,
    costo_alquiler numeric(4,2) DEFAULT 99.8 NOT NULL,
    duracion smallint,
    costo_reemplazo numeric(5,2) DEFAULT 399.8 NOT NULL,
    clasificacion mpaa_rating DEFAULT 'G'::mpaa_rating,
    contenidos_extra text[]
);

CREATE TABLE personal (
    id_personal integer DEFAULT nextval('personal_id_personal_seq'::regclass) NOT NULL,
    nombre character varying(45) NOT NULL,
    apellido character varying(45) NOT NULL,
    direccion character varying(50) NOT NULL,
    distrito character varying(20) NOT NULL,
    id_ciudad smallint NOT NULL,
    codigo_postal character varying(10),
    telefono character varying(20) NOT NULL,
    email character varying(50),
    id_sucursal smallint NOT NULL,
    activo boolean DEFAULT true NOT NULL,
    usuario character varying(16) NOT NULL,
    password character varying(40)
);

CREATE TABLE sucursales (
    id_sucursal integer DEFAULT nextval('sucursal_sucursal_id_seq'::regclass) NOT NULL,
    id_encargado smallint NOT NULL,
    direccion character varying(50) NOT NULL,
    distrito character varying(20) NOT NULL,
    id_ciudad smallint NOT NULL,
    codigo_postal character varying(10),
    telefono character varying(20) NOT NULL
);


--PRIMARY KEYS
ALTER TABLE ONLY actores_de_peliculas
    ADD CONSTRAINT actor_de_pelicula_pkey PRIMARY KEY (id_actor, id_pelicula);

ALTER TABLE ONLY actores
    ADD CONSTRAINT actores_pkey PRIMARY KEY (id_actor);

ALTER TABLE ONLY alquileres
    ADD CONSTRAINT alquileres_pkey PRIMARY KEY (id_pelicula, id_cliente, id_sucursal, id_personal, fecha);

ALTER TABLE ONLY categorias_de_peliculas
    ADD CONSTRAINT categoria_de_pelicula_pkey PRIMARY KEY (id_pelicula, id_categoria);

ALTER TABLE ONLY categorias
    ADD CONSTRAINT categorias_pkey PRIMARY KEY (id_categoria);

ALTER TABLE ONLY ciudades
    ADD CONSTRAINT ciudades_pkey PRIMARY KEY (id_ciudad);

ALTER TABLE ONLY clientes
    ADD CONSTRAINT clientes_pkey PRIMARY KEY (id_cliente);

ALTER TABLE ONLY idiomas_de_peliculas
    ADD CONSTRAINT idioma_de_pelicula_pkey PRIMARY KEY (id_idioma, id_pelicula);

ALTER TABLE ONLY idiomas
    ADD CONSTRAINT idiomas_pkey PRIMARY KEY (id_idioma);

ALTER TABLE ONLY inventario
    ADD CONSTRAINT inventario_pkey PRIMARY KEY (id_pelicula, id_sucursal);

ALTER TABLE ONLY pagos
    ADD CONSTRAINT pagos_pkey PRIMARY KEY (id_pelicula_alquilo, id_cliente_alquilo, id_sucursal_alquilo, id_personal_alquilo, fecha_alquilo, fecha);

ALTER TABLE ONLY paises
    ADD CONSTRAINT paises_pkey PRIMARY KEY (id_pais);

ALTER TABLE ONLY peliculas
    ADD CONSTRAINT peliculas_pkey PRIMARY KEY (id_pelicula);

ALTER TABLE ONLY personal
    ADD CONSTRAINT personal_pkey PRIMARY KEY (id_personal);

ALTER TABLE ONLY sucursales
    ADD CONSTRAINT sucursal_pkey PRIMARY KEY (id_sucursal);


--FOREIGN KEYS
ALTER TABLE ONLY actores_de_peliculas
    ADD CONSTRAINT actor_de_pelicula_actor_id_fkey FOREIGN KEY (id_actor) REFERENCES actores(id_actor) ON UPDATE CASCADE ON DELETE RESTRICT;

ALTER TABLE ONLY actores_de_peliculas
    ADD CONSTRAINT actor_de_pelicula_pelicula_id_fkey FOREIGN KEY (id_pelicula) REFERENCES peliculas(id_pelicula) ON UPDATE CASCADE ON DELETE RESTRICT;

ALTER TABLE ONLY pagos
    ADD CONSTRAINT alquiler_fkey FOREIGN KEY (id_pelicula_alquilo, id_cliente_alquilo, id_sucursal_alquilo, id_personal_alquilo, fecha_alquilo) REFERENCES alquileres(id_pelicula, id_cliente, id_sucursal, id_personal, fecha) ON UPDATE CASCADE ON DELETE RESTRICT;

ALTER TABLE ONLY alquileres
    ADD CONSTRAINT alquileres_alquiler_fkey FOREIGN KEY (id_pelicula, id_sucursal) REFERENCES inventario(id_pelicula, id_sucursal) ON UPDATE CASCADE ON DELETE RESTRICT;

ALTER TABLE ONLY alquileres
    ADD CONSTRAINT alquileres_id_cliente_fkey FOREIGN KEY (id_cliente) REFERENCES clientes(id_cliente) ON UPDATE CASCADE ON DELETE RESTRICT;

ALTER TABLE ONLY alquileres
    ADD CONSTRAINT alquileres_id_personal_fkey FOREIGN KEY (id_personal) REFERENCES personal(id_personal) ON UPDATE CASCADE ON DELETE RESTRICT;

ALTER TABLE ONLY categorias_de_peliculas
    ADD CONSTRAINT categoria_de_pelicula_categoria_id_fkey FOREIGN KEY (id_categoria) REFERENCES categorias(id_categoria) ON UPDATE CASCADE ON DELETE RESTRICT;

ALTER TABLE ONLY categorias_de_peliculas
    ADD CONSTRAINT categoria_de_pelicula_pelicula_id_fkey FOREIGN KEY (id_pelicula) REFERENCES peliculas(id_pelicula) ON UPDATE CASCADE ON DELETE RESTRICT;

ALTER TABLE ONLY ciudades
    ADD CONSTRAINT ciudad_id_pais_fkey FOREIGN KEY (id_pais) REFERENCES paises(id_pais) ON UPDATE CASCADE ON DELETE RESTRICT;

ALTER TABLE ONLY clientes
    ADD CONSTRAINT cliente_ciudad_fkey FOREIGN KEY (id_ciudad) REFERENCES ciudades(id_ciudad) ON UPDATE CASCADE ON DELETE RESTRICT;

ALTER TABLE ONLY clientes
    ADD CONSTRAINT cliente_sucursal_fkey FOREIGN KEY (id_sucursal) REFERENCES sucursales(id_sucursal) ON UPDATE CASCADE ON DELETE RESTRICT;

ALTER TABLE ONLY idiomas_de_peliculas
    ADD CONSTRAINT idioma_de_pelicula_idioma_id_fkey FOREIGN KEY (id_idioma) REFERENCES idiomas(id_idioma) ON UPDATE CASCADE ON DELETE RESTRICT;

ALTER TABLE ONLY idiomas_de_peliculas
    ADD CONSTRAINT idioma_de_pelicula_pelicula_id_fkey FOREIGN KEY (id_pelicula) REFERENCES peliculas(id_pelicula) ON UPDATE CASCADE ON DELETE RESTRICT;

ALTER TABLE ONLY inventario
    ADD CONSTRAINT inventario_id_pelicula_fkey FOREIGN KEY (id_pelicula) REFERENCES peliculas(id_pelicula) ON UPDATE CASCADE ON DELETE RESTRICT;

ALTER TABLE ONLY inventario
    ADD CONSTRAINT inventario_id_sucursal_fkey FOREIGN KEY (id_sucursal) REFERENCES sucursales(id_sucursal) ON UPDATE CASCADE ON DELETE RESTRICT;

ALTER TABLE ONLY pagos
    ADD CONSTRAINT pagos_id_personal_recibe_pago_fkey FOREIGN KEY (id_personal_recibe_pago) REFERENCES personal(id_personal) ON UPDATE CASCADE ON DELETE RESTRICT;

ALTER TABLE ONLY peliculas
    ADD CONSTRAINT pelicula_original_id_idioma_fkey FOREIGN KEY (id_idioma_original) REFERENCES idiomas(id_idioma) ON UPDATE CASCADE ON DELETE RESTRICT;

ALTER TABLE ONLY personal
    ADD CONSTRAINT personal_ciudad_fkey FOREIGN KEY (id_ciudad) REFERENCES ciudades(id_ciudad) ON UPDATE CASCADE ON DELETE RESTRICT;

ALTER TABLE ONLY personal
    ADD CONSTRAINT personal_sucursal_fkey FOREIGN KEY (id_sucursal) REFERENCES sucursales(id_sucursal) ON UPDATE CASCADE ON DELETE RESTRICT;

ALTER TABLE ONLY sucursales
    ADD CONSTRAINT sucursal_id_diudad_fkey FOREIGN KEY (id_ciudad) REFERENCES ciudades(id_ciudad) ON UPDATE CASCADE ON DELETE RESTRICT;
